@@ -332,7 +332,7 @@ class tree:
         self.boundary = pickle.loads(file)
         file.close()
 
-    def export(self,steady=True,apply_distal_resistance=True,gui=True,cylinders=False,make=True,global_edge_size=None):
+    def export(self,steady=True,apply_distal_resistance=True,gui=True,cylinders=False,make=True,global_edge_size=None,splines=False,splines_file_path='',spline_sample_points=100):
         if cylinders:
             pv_data = []
             models = self.show(show=False)
@@ -341,7 +341,7 @@ class tree:
             merge = pv_data[0].merge(pv_data[1:])
             return merge
         interp_xyz,interp_r,interp_n,frames,branches,interp_xyzr = get_interpolated_sv_data(self.data)
-        points,radii,normals    = sv_data(interp_xyzr,interp_r,radius_buffer=self.radius_buffer)
+        #points,radii,normals    = sv_data(interp_xyzr,interp_r,radius_buffer=self.radius_buffer)
         if steady:
             time = [0, 1]
             flow = [self.data[0,22], self.data[0,22]]
@@ -359,7 +359,18 @@ class tree:
             options = file_options(num_caps,time=time,flow=flow,gui=gui,distal_resistance=R)
             if global_edge_size is None:
                 options.set_mesh_options(global_edge_size=global_edge_size)
+            points,radii,normals    = sv_data(interp_xyzr,interp_r,radius_buffer=self.radius_buffer)
             build(points,radii,normals,options)
+        if splines:
+            spline_file = open(splines_file_path+'b_splines.txt','w+')
+            for ind,spline in enumerate(interp_xyzr):
+                spline_file.write('Vessel: {}, Number of Points: {}\n\n'.format(ind,spline_sample_points))
+                n = np.linspace(0,1,spline_sample_points)
+                spline_data = splev(n,spline[0])
+                for j in range(spline_sample_points):
+                    spline_file.write('{}, {}, {}\n'.format(spline_data[0][j],spline_data[1][j],spline_data[2][j]))
+                spline_file.write('\n')
+            spline_file.close()
         return interp_xyz,interp_r,interp_xyzr
 
     def export_truncated(self,steady=True,radius=None,indicies=None,apply_distal_resistance=True,gui=True,cylinders=False,make=True):
@@ -609,6 +620,99 @@ class tree:
         geom[:,6]   = self.data[:,20]
         geom[:,7]   = self.data[:,21]
         np.savetxt(outdir+os.sep+"geom.csv",geom,delimiter=",")
+
+    def export_1d_simulation(self,steady=True,outdir=None,folder="1d_tmp",number_cariac_cycles=1,num_points=100):
+        _,_,interp_xyzr = self.export(make=False)
+        # Make Centerline Approximation Polydata
+        #branches = get_branches(self.data)
+        #centerline_ids = []
+        #for id,branch in enumerate(branches):
+        #    branch_ids = []
+        #    for seg in branch
+        def make_points(x,y,z):
+            """Helper to make XYZ points"""
+            return np.column_stack((x, y, z))
+
+        def lines_from_points(points):
+            """Given an array of points, make a line set"""
+            poly = pv.PolyData()
+            poly.points = points
+            cells = np.full((len(points) - 1, 3), 2, dtype=np.int_)
+            cells[:, 1] = np.arange(0, len(points) - 1, dtype=np.int_)
+            cells[:, 2] = np.arange(1, len(points), dtype=np.int_)
+            poly.lines = cells
+            return poly
+
+        polys = []
+        for ind,spline in enumerate(interp_xyzr):
+            n = np.linspace(0,1,num_points)
+            spline_data = splev(n,spline[0])
+            spline_data_normal = splev(n,spline[0],der=1)
+            points = make_points(spline_data[0],spline_data[1],spline_data[2])
+            normal = make_points(spline_data_normal[0],spline_data_normal[1],spline_data_normal[2])
+            normal = normal/np.linalg.norm(normal,axis=1)
+            poly_line = lines_from_points(points)
+            poly_line['VesselId'] = np.ones(num_points,dtype=int)*ind
+            poly_line['MaximumInscribedSphereRadius'] = spline_data[3]
+            poly_line['CenterlineSectionArea'] = np.pi*spline_data[3]**2
+            poly_line.point_data.set_vector(normal,'CenterlineSectionNormal')
+            polys.append(poly_line)
+
+        for ind in range(len(polys)):
+            cent_ids = np.zeros((polys[ind],len(polys)),dtype=int)
+            polys[ind].point_data.set_array(cent_ids,'CenterlineId')
+            polys[ind].point_data['CenterlineId'][:,ind] = 1
+
+        bifucation_point_ids = [] # polys ind index, polys jnd index, polys jnd point index
+        for ind in range(1,len(polys)):
+            for jnd in range(len(polys)):
+                if jnd == ind:
+                    continue
+                closest_point_id = polys[jnd].find_closest_point(polys[ind].points[0])
+                closest_point    = polys[jnd].points[closest_point_id]
+                dist             = np.linalg.norm(polys[ind].points[0] - closest_point)
+                if dist < polys[ind].point_data['Radius'][0]:
+                    polys[jnd].point_data['CenterlineId'][0:closest_point_id+1,ind] = 1
+                    bifurcation_point_ids.append([ind,jnd,closest_point_id])
+
+        # Determine Branch Temp Ids
+        branch_tmp_count = 0
+        for ind in range(len(polys)):
+            tmp_split = []
+            for bif in bifurcation_point_ids:
+                if bif[1] == ind:
+                    tmp_split.append(bif[2])
+            tmp_split.sort()
+            tmp_split.insert(0,0)
+            tmp_split.append(None)
+            branch_tmp_ids = np.zeros(len(polys.points.shape[0]),dtype=int)
+            for i in range(1,len(tmp_split)):
+                branch_tmp_ids[tmp_split[i-1]:tmp_split[i]] = branch_tmp_count
+                branch_tmp_count += 1
+            polys[ind].point_data['BranchIdTmp'] = branch_tmp_ids
+
+        for ind in range(len(polys)):
+            tmp_bif = []
+            tmp_bif_pt = []
+            for bif in bifurcation_point_ids:
+                if bif[1] == ind:
+                    rad = polys[ind].point_data['MaximumInscribedSphereRadius'][bif[2]]
+                    pt  = polys[ind].points[bif[2]]
+                    b   = np.argwhere(np.linalg.norm(polys[ind].points - pt,axis=1)<rad).flatten().tolist()
+                    b_pt = b.pop(b.index(bif[2]))
+                    tmp_bif.extend(b)
+                    tmp_bif_pt.append(b_pt)
+            tmp_bif = np.array(tmp_bif)
+            tmp_bif_pt = np.array(tmp_bif_pt)
+            bifurcation_id_tmp = np.ones(len(polys.points.shape[0]),dtype=int)*-1
+            bifurcation_id_tmp[tmp_bif] = 2
+            bifurcation_id_tmp[tmp_bif_pt] = 1
+            polys[ind].point_data['BifurcationIdTmp'] = bifurcation_id_tmp
+
+        # Assign branch ids
+        # np.argwhere(polys[ind].point_data['BifurcationIdTmp']==)
+        return polys
+
     def collision_free(self,outside_vessels,radius_buffer=0.01):
         return no_outside_collision(self,outside_vessels,radius_buffer=radius_buffer)
 
